<?php
/**
 * Plugin Name: Tickera - Create order and tickets from imported CSV
 * Plugin URI: https://tickera.com/
 * Description: Create order and tickets from imported CSV ( Custom add-on for Tickera )
 * Version: 2.0
 * Author: Tickera
 * Author URI: https://tickera.com/
 * Developer: Tickera
 * Developer URI: https://tickera.com/
 */

if ( ! defined( 'ABSPATH' ) )
    exit; // Exit if accessed directly

if ( ! class_exists( 'TC_Import_Order_Tickets_From_CSV' ) ) {

    class TC_Import_Order_Tickets_From_CSV {

		var $version = '2.0';
		var $title = 'Tickera - Create order and tickets from imported CSV';
		var $name = 'tc-custom-create-orders-and-tickets-from-csv';
		var $dir_name = 'tc-custom-create-orders-and-tickets-from-csv';
		var $location = 'plugins';
		var $plugin_dir = '';
		var $plugin_url = '';

        function __construct() {
			self::init_vars();
            add_filter( 'tc_settings_new_menus', array( &$this, 'tc_settings_new_menus_custom_csv_import' ) );
            add_action( 'tc_settings_menu_tickera_custom_import_csv', array( &$this, 'tc_settings_menu_tickera_custom_import_csv_show_page' ) );
            add_action( 'admin_init', array( &$this, 'tc_custom_import_data' ), 0 );
        }

		function init_vars() {

			if ( defined( 'WP_PLUGIN_URL' ) && defined( 'WP_PLUGIN_DIR' ) && file_exists( WP_PLUGIN_DIR . '/' . $this->dir_name . '/' . basename( __FILE__ ) ) ) {
				$this->location = 'subfolder-plugins';
				$this->plugin_dir = WP_PLUGIN_DIR . '/' . $this->dir_name . '/';
				$this->plugin_url = plugins_url( '/', __FILE__ );

			} elseif ( defined( 'WP_PLUGIN_URL' ) && defined( 'WP_PLUGIN_DIR' ) && file_exists( WP_PLUGIN_DIR . '/' . basename( __FILE__ ) ) ) {
				$this->location = 'plugins';
				$this->plugin_dir = WP_PLUGIN_DIR . '/';
				$this->plugin_url = plugins_url( '/', __FILE__ );

			} elseif ( is_multisite() && defined( 'WPMU_PLUGIN_URL' ) && defined( 'WPMU_PLUGIN_DIR' ) && file_exists( WPMU_PLUGIN_DIR . '/' . basename( __FILE__ ) ) ) {
				$this->location = 'mu-plugins';
				$this->plugin_dir = WPMU_PLUGIN_DIR;
				$this->plugin_url = WPMU_PLUGIN_URL;
			} else {
				wp_die( sprintf( __( 'There was an issue determining where %s is installed. Please reinstall it.', 'tc' ), $this->title ) );
			}
		}

		function tc_settings_new_menus_custom_csv_import( $settings_tabs ) {
			$settings_tabs[ 'tickera_custom_import_csv' ] = __( 'Import CSV', 'tc' );
			return $settings_tabs;
		}

		function tc_settings_menu_tickera_custom_import_csv_show_page() {
			require_once $this->plugin_dir . 'includes/admin-pages/settings-tickera_custom_import_csv.php';
		}

        function tc_custom_import_data() {

            if ( isset( $_POST[ 'tc_custom_import_csv' ] ) ) {

                @error_reporting( E_ERROR );
                @set_time_limit( 0 );
                @ini_set( 'max_input_time', 3600 * 3 );
                @ini_set( 'max_execution_time', 3600 * 3 );

                if ( $_FILES[ 'tc_csv_import_file' ][ 'size' ] > 0 && 'text/csv' == stripslashes( $_FILES[ 'tc_csv_import_file' ][ 'type' ] ) ) {

                    // Get the csv file
                    $file = $_FILES[ 'tc_csv_import_file' ][ 'tmp_name' ];
                    $handle = fopen( $file, "r" );

                    $row = 0;

                    while ( ! feof( $handle ) ) {

                        $data = fgetcsv( $handle, 100000, ",", "'" );

                        if ( $row ) {

                            $fields = apply_filters( 'tc_import_order_ticket_fields', [
                                'first_name' => $data[0],
                                'last_name' => $data[1],
                                'order_id' => $data[2],
                                'email' => $data[3]
                            ] );

                            self::tc_custom_create_order_from_csv( (int) $_POST[ 'tc_ticket_type_id' ], $fields );
                        }

                        $row++;
                    }

                    fclose( $handle );
                }
            }
        }

        function tc_custom_create_order_from_csv( $tc_ticket_type_id, $fields ) {

            global $tc;

            if ( ! session_id() ) {
                @session_start();
            }

            $order_id = isset( $fields['order_id'] ) ? $fields['order_id'] : '';

            // Order ID
            if ( ! $order_id ) {
                $order_id = $tc->generate_order_id();
            }

            $gateway_class = 'TC_Gateway_Free_Orders';
            $free_orders_gateway = new TC_Gateway_Free_Orders();

            // Cart Contents
            $cart = array();
            $cart[ $tc_ticket_type_id ] = 1;
            $cart_contents = $cart;

            // Buyer Data
            $buyer_data = array();
            $buyer_data[ 'first_name_post_meta' ] = $fields['first_name'];
            $buyer_data[ 'last_name_post_meta' ] = $fields['last_name'];
            $buyer_data[ 'email_post_meta' ] = $fields['email'];

            // Owner Data
            $owner_data = array();
            $owner_data[ 'first_name_post_meta' ] = $fields['first_name'];
            $owner_data[ 'last_name_post_meta' ] = $fields['last_name'];
            $owner_data[ 'email_post_meta' ] = $fields['email'];
            $owner_data[ 'ticket_type_id_post_meta' ] = $tc_ticket_type_id;

            // Cart Info
            $cart_info = array();
            $cart_info[ 'coupon_code' ] = '';
            $cart_info[ 'total' ] = 0;
            $cart_info[ 'currency' ] = $tc->get_cart_currency();
            $cart_info[ 'buyer_data' ] = $buyer_data;
            $cart_info[ 'owner_data' ] = $owner_data;
            $cart_info[ 'gateway_class' ] = $gateway_class;
            $cart_info[ 'gateway' ] = $free_orders_gateway->plugin_name;
            $cart_info[ 'gateway_admin_name' ] = $free_orders_gateway->admin_name;

            // Payment Info
            $payment_info = array();
            $payment_info[ 'currency' ] = $tc->get_cart_currency();
            $payment_info = $free_orders_gateway->save_payment_info( $payment_info );

            self::tc_custom_csv_create_order_internal( $order_id, $cart_contents, $cart_info, $payment_info, true );
        }

        function tc_custom_csv_create_order_internal( $order_id, $cart_contents, $cart_info, $payment_info, $paid ) {

            global $tc;

            if ( empty( $order_id ) ) {
                $order_id = $tc->generate_order_id();

            } elseif ( self::get_order( $order_id ) ) {
                // Don't continue if the order exists
                return false;
            }

            $cart_total = $cart_info[ 'total' ];
            $fraud = $tc->check_for_total_paid_fraud( $payment_info[ 'total' ], $cart_total );
            $user_id = get_current_user_id();

            // Insert post type
            $status = ( $paid ? ( $fraud ? 'order_fraud' : 'order_paid' ) : 'order_received' );

            $order = array();
            $order[ 'post_title' ] = $order_id;
            $order[ 'post_name' ] = $order_id;
            $order[ 'post_content' ] = serialize( $cart_contents );
            $order[ 'post_status' ] = $status;
            $order[ 'post_type' ] = 'tc_orders';

            if ( $user_id != 0 ) {
                $order[ 'post_author' ] = $user_id;
            }

            $post_id = wp_insert_post( $order );

            // Cart Contents
            add_post_meta( $post_id, 'tc_cart_contents', $cart_contents );

            // Cart Info
            add_post_meta( $post_id, 'tc_cart_info', $cart_info ); //save row data - buyer and ticket owners data, gateway, total, currency, coupon code, etc.

            // Payment Info
            add_post_meta( $post_id, 'tc_payment_info', $payment_info ); //transaction_id, total, currency, method

            // Order Date & Time
            add_post_meta( $post_id, 'tc_order_date', time() );

            // Order Paid Time
            add_post_meta( $post_id, 'tc_paid_date', ( $paid ) ? time() : '' ); //empty means not yet paid

            // Event(s) - could be more events at once since customer may have tickets from more than one event in the cart
            add_post_meta( $post_id, 'tc_parent_event', $tc->get_cart_events( $cart_contents ) );
            add_post_meta( $post_id, 'tc_event_creators', $tc->get_events_creators( $cart_contents ) );

            // Discount Code
            add_post_meta( $post_id, 'tc_paid_date', ( $paid ) ? time() : '' );

            // Save Ticket Owner(s) data
            $owner_data = $cart_info[ 'owner_data' ];
            $owner_records = array();
            $different_ticket_types = array_keys( array( $owner_data[ 'ticket_type_id_post_meta' ] ) );
            $owner_record_num = 1;

            $metas[ 'first_name' ] = $owner_data[ 'first_name_post_meta' ];
            $metas[ 'last_name' ] = $owner_data[ 'last_name_post_meta' ];
            $metas[ 'email' ] = $owner_data[ 'email_post_meta' ];
            $metas[ 'ticket_type_id' ] = $owner_data[ 'ticket_type_id_post_meta' ];

            if ( apply_filters( 'tc_use_only_digit_order_number', false ) == true ) {
                $metas[ 'ticket_code' ] = apply_filters( 'tc_ticket_code', $order_id );

            } else {
                $metas[ 'ticket_code' ] = apply_filters( 'tc_ticket_code', $order_id );
            }

            $arg = array(
                'post_author' => isset( $user_id ) ? $user_id : '',
                'post_parent' => $post_id,
                'post_excerpt' => ( isset( $excerpt ) ? $excerpt : '' ),
                'post_content' => ( isset( $content ) ? $content : '' ),
                'post_status' => 'publish',
                'post_title' => ( isset( $title ) ? $title : '' ),
                'post_type' => 'tc_tickets_instances',
            );

            $owner_record_id = @wp_insert_post( $arg, true );
            foreach ( $metas as $meta_name => $mata_value ) {
                update_post_meta( $owner_record_id, $meta_name, $mata_value );
            }

            $ticket_type_id = get_post_meta( $owner_record_id, 'ticket_type_id', true );
            $ticket_type = new TC_Ticket( $ticket_type_id );
            $event_id = $ticket_type->get_ticket_event();

            update_post_meta( $owner_record_id, 'event_id', $event_id );

            // Send order status email to the customer
            $payment_class_name = $cart_info[ 'gateway_class' ];
            $payment_gateway = new TC_Gateway_Free_Orders();
            do_action( 'tc_order_created', $order_id, $status, $cart_contents, $cart_info, $payment_info );
            unset( $_SESSION[ 'tc_order' ] );
            return $order_id;
        }

        function get_order( $order_id ) {

            $id = ( is_int( $order_id ) ) ? $order_id : tc_get_order_id_by_name( $order_id );

            if ( empty( $id ) )
                return false;

            $order = get_post( $id->ID );

            if ( ! $order )
                return false;

            $meta = get_post_custom( $id->ID );

            foreach ( $meta as $key => $val )
                $order->{$key} = maybe_unserialize( $meta[ $key ][ 0 ] );

            return $order;
        }
    }

    $TC_Import_Order_Tickets_From_CSV = new TC_Import_Order_Tickets_From_CSV();
}
